==> spiped_1.4.1.orig.tar.gz: spiped-1.4.1/libcperciva/util/daemonize.c <==
#include <errno.h>
#include <stdio.h>
#include <unistd.h>

#include "noeintr.h"
#include "warnp.h"

#include "daemonize.h"

/**
 * daemonize(spid):
 * Daemonize and write the process ID in decimal to a file named ${spid}.
 * The parent process will exit only after the child has written its pid.
 * On success, the child will return 0; on failure, the parent will return
 * -1.
 */
int
daemonize(const char * spid)
{
	FILE * f;
	int fd[2];
	char dummy = 0;

	/*
	 * Create a pipe for the child to notify the parent when it has
	 * finished daemonizing.
	 */
	if (pipe(fd)) {
		warnp("pipe");
		goto err0;
	}

	/*
	 * Fork into the parent process (which waits for a poke and exits)
	 * and the child process (which keeps going).
	 */
	switch (fork()) {
	case -1:
		/* Fork failed. */
		warnp("fork");
		goto err2;
	case 0:
		/* In child process. */
		break;
	default:
		/*
		 * In parent process.  Close write end of pipe so that if the
		 * client dies we will notice the pipe being reset.
		 */
		while (close(fd[1])) {
			if (errno == EINTR)
				continue;
			warnp("close");
			goto err1;
		}
		do {
			switch (read(fd[0], &dummy, 1)) {
			case -1:
				/* Error in read. */
				break;
			case 0:
				/* EOF -- the child died without poking us. */
				goto err1;
			case 1:
				/* We have been poked by the child.  Exit. */
				_exit(0);
			}

			/* Anything other than EINTR is bad. */
			if (errno != EINTR) {
				warnp("read");
				goto err1;
			}
		} while (1);
	}

	/* Set ourselves to be a session leader. */
	if (setsid() == -1) {
		warnp("setsid");
		goto die;
	}

	/* Write out our pid file. */
	if ((f = fopen(spid, "w")) == NULL) {
		warnp("fopen(%s)", spid);
		goto die;
	}
	if (fprintf(f, "%d", getpid()) < 0) {
		warnp("fprintf");
		goto die;
	}
	if (fclose(f)) {
		warnp("fclose");
		goto die;
	}

	/* Tell the parent to suicide. */
	if (noeintr_write(fd[1], &dummy, 1) == -1) {
		warnp("write");
		goto die;
	}

	/* Close the pipe. */
	while (close(fd[0])) {
		if (errno == EINTR)
			continue;
		warnp("close");
		goto die;
	}
	while (close(fd[1])) {
		if (errno == EINTR)
			continue;
		warnp("close");
		goto die;
	}

	/* Success! */
	return (0);

err2:
	close(fd[1]);
err1:
	close(fd[0]);
err0:
	/* Failure! */
	return (-1);

die:
	/*
	 * We're in the child and something bad happened; the parent will be
	 * notified when we die thanks to the pipe being closed.
	 */
	_exit(0);
}

==> spiped_1.6.4.orig.tar.gz: spiped-1.6.4/libcperciva/util/daemonize.c <==
#include <stdint.h>
#include <stdio.h>
#include <unistd.h>

#include "ipc_sync.h"
#include "warnp.h"

#include "daemonize.h"

/* Write the process' pid to a file called ${spid}. */
static int
writepid(const char * spid)
{
	FILE * f;

	/* Write our pid to a file. */
	if ((f = fopen(spid, "w")) == NULL) {
		warnp("fopen(%s)", spid);
		goto err0;
	}
	if (fprintf(f, "%jd", (intmax_t)getpid()) < 0) {
		warnp("fprintf");
		goto err1;
	}
	if (fclose(f)) {
		warnp("fclose");
		goto err0;
	}

	/* Success! */
	return (0);

err1:
	if (fclose(f))
		warnp("fclose");
err0:
	/* Failure! */
	return (-1);
}

/**
 * daemonize(spid):
 * Daemonize and write the process ID in decimal to a file named ${spid}.
 * The parent process will exit only after the child has written its pid.
 * On success, the child will return 0; on failure, the parent will return
 * -1.
 */
int
daemonize(const char * spid)
{
	struct ipc_sync * IS;

	/*
	 * Create a pipe for the child to notify the parent when it has
	 * finished daemonizing.
	 */
	if ((IS = ipc_sync_init()) == NULL)
		goto err0;

	/*
	 * Fork into the parent process (which waits for a poke and exits)
	 * and the child process (which keeps going).
	 */
	switch (fork()) {
	case -1:
		/* Fork failed. */
		warnp("fork");
		goto err1;
	case 0:
		/* In child process. */
		break;
	default:
		/* In parent process; wait for the child to start. */
		if (ipc_sync_wait(IS))
			goto err1;

		/*
		 * Free resources associated with ${IS}.  These would be
		 * released by the following _exit(), but we're explicitly
		 * releasing them for the benefit of leak checkers.
		 */
		if (ipc_sync_done(IS))
			goto err0;

		/* We have been poked by the child.  Exit. */
		_exit(0);
	}

	/* Set ourselves to be a session leader. */
	if (setsid() == -1) {
		warnp("setsid");
		goto die;
	}

	/* Write out our pid file. */
	if (writepid(spid))
		goto die;

	/* Tell the parent that we've started. */
	if (ipc_sync_signal(IS))
		goto die;

	/* Clean up. */
	if (ipc_sync_done(IS))
		goto die;

	/* Success! */
	return (0);

err1:
	ipc_sync_done(IS);
err0:
	/* Failure! */
	return (-1);

die:
	/*
	 * We're in the child and something bad happened; the parent will be
	 * notified when we die thanks to the pipe being closed.
	 */
	_exit(0);
}
